Modules
- sigma
- The submodules that would be useful to have in the lower-level
sigma-proofscrate are for now included as submodules of a localsigmamodule.
Structs
- CodeGen
- The main struct to handle code generation for this macro.
- Sigma
Comp Spec - The
SigmaCompSpecstruct is the result of parsing the macro input. - Tagged
Point - A
TaggedPointis anIdentrepresenting aPoint, preceded by zero or more of the following tags:cind,const,vec - Tagged
Scalar - A
TaggedScalaris anIdentrepresenting aScalar, preceded by zero or more of the following tags:pub,rand,vec
Enums
- Tagged
Ident - A
TaggedIdentcan be either aTaggedScalaror aTaggedPoint
Functions
- apply_
transformations - Apply all of the compiler transformations.
- enforce_
disjunction_ invariant - Transform the
StatementTreeso that it satisfies the disjunction invariant. - sigma_
compiler_ core - The main function of this macro.
Type Aliases
- Tagged
VarDict - A
TaggedVarDictis a dictionary of the available variables, mapping the string version ofIdents toTaggedIdent, which includes their type (ScalarorPoint)