pub struct SigmaCompSpec {
pub proto_name: Ident,
pub group_name: Ident,
pub vars: TaggedVarDict,
pub statements: StatementTree,
}Expand description
The SigmaCompSpec struct is the result of parsing the macro
input.
Fields
proto_name: IdentAn identifier for the name of the zero-knowledge protocol being defined
group_name: IdentAn identifier for the mathematical
PrimeGroup
being used (if none is specified, it is assumed there is a
default type called G in scope that implements the
PrimeGroup
trait)
vars: TaggedVarDictA TaggedVarDict mapping variable names to their types
(Scalar or Point) and tags (e.g., rand, pub, vec,
cind, const)
statements: StatementTreeA StatementTree representing the statements provided in the
macro invocation that are to be proved true in zero knowledge